/*
    Title:    AVR-GCC test program #10 for the STK200 eva board
    Author:   Volker Oth
    Date:     1/2000
    Purpose:  demonstrates how to access external memory.
    needed    
    Software: AVR-GCC
    needed
    Hardware: ATS90S8515 STK200 board
    Note:     To contact me, mail to
              volkeroth(at)gmx.de
*/

#include <io.h>
#include <stddef.h>


#define EDATA_SIZE 0x7e00
#define IDATA __attribute__ ((section (".data")))
#define EDATA __attribute__ ((section (".edata")))
#define enable_xram(ws) { sbi(MCUCR, SRE); if (ws) sbi(MCUCR,SRW); }


typedef unsigned char  u08;
typedef unsigned short u16;
typedef u08 ext_mem_t[EDATA_SIZE];


// make the linker script definitions available
extern size_t __edata_start;
extern size_t __edata_end;


ext_mem_t EDATA ext_mem;
ext_mem_t *ext_mem_ptr;


int main(void)
{
    u08 *byte_ptr;
    u08 chunk;
    u16 i;

    outp(0xff,DDRB);                /* use all pins on PortB for output */    
    enable_xram(1);

    ext_mem_ptr = (ext_mem_t*)&__edata_start;

    // check external RAM
    for (;;)
        for (i=0; i<EDATA_SIZE; i++) {
            chunk = i >> 7;         /* divide by 128 */
            // output memory chunk at Port B
            outp(~chunk, DDRB);
            // construct byte pointer to an absolute address
            byte_ptr = (void*)(i+(u16)&__edata_start);
            // write 01010101b to memory
            *byte_ptr = 0x55;        
            if (*ext_mem_ptr[i] != 0xaa)
                // freeze
                for (;;) {}
            // write 10101010b to memory
            ext_mem[i] = 0xaa;
            if (ext_mem[i] != 0xaa)
                // freeze
                for (;;) {}
        }
}
